import { Controller, Post, Body, Req, HttpCode, HttpStatus, Get, Query, Param, Patch, Delete } from '@nestjs/common';
import { OrderService } from './orders.service';
import { CreateOrderDto } from './dto/create-order.dto';
import { Constants } from "src/utils/constants";
import { AuthGuard } from "src/core/guards/auth.guard";
import { UseGuards } from "@nestjs/common/decorators/core/use-guards.decorator";
import { AdminAuthGuard } from "src/core/guards/admin-auth.guard";
import { CreateOrderByAdminDto } from "./dto/create-order-by-admin.dto";
import { CreateOrderByGuestDto } from "./dto/create-order-by-guest.dto";
import { UpdateOrderDto } from "./dto/update-order.dto";

@Controller({ path: 'orders', version: Constants.API_VERSION_1 })
export class OrderController {
    constructor(private readonly ordersService: OrderService) { }

    @Post()
    @UseGuards(AuthGuard)
    create(@Body() createOrderDto: CreateOrderDto, @Req() req: any) {
        return this.ordersService.create(createOrderDto, req.user);
    }

    @Post('/admin')
    @UseGuards(AdminAuthGuard)
    createOrderByAdmin(@Body() createOrderByAdminDto: CreateOrderByAdminDto, @Req() req: any) {
        return this.ordersService.createOrderByAdmin(createOrderByAdminDto, req.user);
    }

    @Post('/guest')
    createOrderByGuest(@Body() dto: CreateOrderByGuestDto) {
        return this.ordersService.createOrderByGuest(dto);
    }

    @Post('/validate-shipping')
    @HttpCode(HttpStatus.OK)
    validateShipping(@Body() dto: any) {
        return this.ordersService.validateShipping(dto);
    }

    @Post('update-tracking-no')
    @HttpCode(HttpStatus.OK)
    updateOrderTracking(@Body() dto: any) {
        return this.ordersService.updateOrderTracking(dto);
    }

    @Get()
    @UseGuards(AdminAuthGuard)
    async findAll(
        @Query('limit') limit: number,
        @Query('page') page: number,
        @Query('paymentStatus') paymentStatus?: string,
        @Query('deliveryStatus') deliveryStatus?: string,
        @Query('startDate') startDate?: Date,
        @Query('endDate') endDate?: Date,
        @Query('customerName') customerName?: string,
        @Query('orderId') orderId?: string
    ) {
        return await this.ordersService.findAll(limit, page, paymentStatus, deliveryStatus, startDate, endDate, customerName, orderId);
    }

    @Get('/filter')
    @UseGuards(AuthGuard)
    async findOrderByFilter(
        @Req() authUser: any,
        @Query('limit') limit: string,
        @Query('page') page: string,
        @Query('tabStatus') tabStatus: string
    ) {
        const pageParam = parseInt(page, 10) || 1;
        const limitParam = parseInt(limit, 10) || 10;
        return await this.ordersService.findOrderByFilter(authUser, limitParam, pageParam, tabStatus);
    }

    @Get('/profit')
    async getOrderProfit(
        @Query('limit') limit: number,
        @Query('page') page: number,
        @Query('startDate') startDate?: Date,
        @Query('endDate') endDate?: Date,
        @Query('orderId') orderId?: string
    ) {
        return await this.ordersService.orderProfitService(limit, page, startDate, endDate, orderId);
    }

    @Get('/get-orderid')
    @UseGuards(AdminAuthGuard)
    getOrderId() {
        return this.ordersService.getOrderId();
    }

    @Get('/check-orderid/:id')
    checkOrderId(@Param('id') id: string) {
        return this.ordersService.checkOrderId(id);
    }

    @Get('/:id')
    @UseGuards(AuthGuard)
    async findOne(@Param('id') id: string) {
        return await this.ordersService.findOne(id);
    }

    @Get('/order-details/:id')
    async detailsWithoutGuard(@Param('id') id: string) {
        return await this.ordersService.findOne(id);
    }

    @Patch('/shipping-label/:id')
    createShippingLabel(@Param('id') id: string, @Body() dto: any) {
        return this.ordersService.createShippingLabel(id, dto);
    }

    @Patch('/:id')
    update(@Param('id') id: string, @Body() dto: UpdateOrderDto) {
        return this.ordersService.update(id, dto);
    }

    @Delete(':id')
    @UseGuards(AdminAuthGuard)
    async remove(@Param('id') id: string) {
        return await this.ordersService.remove(id);
    }
}
